@@ -37,14 +37,17 @@ short det_sket_sig = FALSE;
 struct message *rootmess;
 struct config_struct *source = NULL, 
                      *target = NULL;
-
+char **mailer = NULL;
 char	* recipient   = NULL,		/* Recipient	*/
 	* sender      = NULL;		/* Sender 	*/
 static char * rmx     = NULL;		/* Recipient's mx */
-
+int   want_unix_from = 0;
+#ifdef SMTP
+int smtp_client = 0;
+#endif
 #ifdef	sun
 extern	char	* optarg;
-#elif	ultrix
+#elif	defined(ultrix) || defined(_AUX_SOURCE)
 extern	char	* optarg;
 #endif
 #ifdef DEBUG
@@ -61,13 +64,14 @@ int	syslog_facility	= LOG_FACILITY;
 
 int	header_logging	= 0;
 int	syslog_logging	= 0;
-int	spawn_sendmail	= 0;
+int	spawn_mailer	= 0;
 int     pseudo_route    = 0;
 
 int	in_fd		= 0;
 FILE	* out_fd	= stdout;
 FILE	* char_fd	= NULL;
 FILE	* conf_fd	= NULL;
+off_t   pz;
 
 int	main(int argc, char *argv[])
 {
@@ -76,10 +80,18 @@ int	main(int argc, char *argv[])
   int	c;
   int   no_run = FALSE;
 
-#ifdef	ultrix
-  while((c=getopt(argc, argv, "s:r:x:i:o:e:c:B:H:F:C:S:T:lhfpgd")) >= 0) {
+
+#ifdef	HAVE_GETPAGESIZE
+	pz = (off_t) getpagesize();
+#elif	defined(hpux) || defined(_AUX_SOURCE)
+	pz = 4096;
 #else
-  while((c=getopt(argc, argv, "s:r:x:l:i:o:e:c:B:H:F:C:S:T:hfpgd")) >= 0) {
+	pz = sysconf(_SC_PAGESIZE);
+#endif
+#ifdef	ultrix
+  while((c=getopt(argc, argv, "s:r:x:i:m:o:e:c:B:H:F:C:S:T:l:h:fpgduvn")) >= 0) {
+#else
+  while((c=getopt(argc, argv, "s:r:x:l:i:m:o:e:c:B:H:F:C:S:T:h:f:pgduvn")) >= 0) {
 #endif
 	switch (c) {
 	case	's':	/* Sender's name */
@@ -91,7 +103,23 @@ int	main(int argc, char *argv[])
 	case	'x':	/* Recipient's mail exchanger address */
 		rmx	= optarg;
 		break;
-	case	'l':	/* syslog facility */
+	case	'l':	/* syslog level */
+		if (optarg == NULL) {
+			/* Just increment */
+			syslog_logging++;
+			break;
+		}
+		else
+		  switch(*optarg) {
+		  case '1': syslog_logging = 1; break;
+		  case '2': syslog_logging = 2; break;
+		  case '3': syslog_logging = 3; break;
+		  case '4': syslog_logging = 4; break;
+		  case '5': syslog_logging = 5; break;
+		  default:  syslog_logging++ ; break;
+		  }
+		break;
+	      case 'f':
 #ifndef	ultrix
 		if (optarg == NULL) {
 			/* Just increment */
@@ -111,18 +139,42 @@ int	main(int argc, char *argv[])
 		case	'6':	syslog_facility	= LOG_LOCAL6;	break;
 		case	'7':	syslog_facility	= LOG_LOCAL7;	break;
 		default:	
-			sprintf(ebuf,"Invalid parameter to -l: %s",optarg);
+			sprintf(ebuf,"Invalid parameter to -f: %s",optarg);
 			logger(LOG_WARNING,ebuf);
 		}
 #endif
-		syslog_logging++;
 		break;
 	case	'h':	/* Log in header */
-		header_logging++;
+		if (optarg == NULL) {
+			/* Just increment */
+			header_logging++;
+			break;
+		}
+		else
+		  switch(*optarg) {
+		  case '1': header_logging = 1; break;
+		  case '2': header_logging = 2; break;
+		  case '3': header_logging = 3; break;
+		  case '4': header_logging = 4; break;
+		  case '5': header_logging = 5; break;
+		  default:  header_logging++ ; break;
+		  }
 		break;
-	case	'f':	/* Fork mode (sendmail spawning) */
-		spawn_sendmail	= 1;
+	      case 'm':
+		mailer = get_mailer(optarg);
+		if (mailer == NULL) {
+		        sprintf(ebuf, "Invalid mailer specification", optarg);
+			logger(LOG_ERR, ebuf);
+			fprintf(stderr, "Emil: %s\n", ebuf);
+			exit(EX_USAGE);
+		}
+		spawn_mailer = 1;
 		break;
+#ifdef SMTP
+        case    'n':    /* SMTP Client */
+		smtp_client = 1;
+		break;
+#endif
         case    'p':    /* Add loop back pseudo route */
 		pseudo_route = 1;
 		break;
@@ -141,6 +193,7 @@ int	main(int argc, char *argv[])
 		in_fd = open(optarg,O_RDONLY,0);
 		if (in_fd < 0) {
 			logger(LOG_ERR,"Cannot open input file");
+			fprintf(stderr, "Emil: Cannot open input file\n");
 			exit(EX_NOINPUT);
 		}
 		break;
@@ -148,6 +201,7 @@ int	main(int argc, char *argv[])
 		out_fd	= fopen(optarg,"w");
 		if (out_fd == NULL) {
 			logger(LOG_ERR,"Cannot open output file");
+			fprintf(stderr, "Emil: Cannot open output file\n");
 			exit(EX_CANTCREAT);
 		}
 		break;
@@ -155,6 +209,7 @@ int	main(int argc, char *argv[])
 		conf_fd	= fopen(optarg,"r");
 		if (conf_fd == NULL) {
 			logger(LOG_ERR,"Cannot open emil.cf");
+			fprintf(stderr, "Emil: Cannot open emil.cf\n");
 			exit(EX_OSFILE);
 		}
 		break;
@@ -162,6 +217,7 @@ int	main(int argc, char *argv[])
 		char_fd	= fopen(optarg,"r");
 		if (char_fd == NULL) {
 			logger(LOG_ERR,"Cannot open charsets file");
+			fprintf(stderr, "Emil: Cannot open charsets file\n");
 			exit(EX_OSFILE);
 		}
 		break;
@@ -195,12 +251,41 @@ int	main(int argc, char *argv[])
 		  source = (struct config_struct *)Yalloc(sizeof(struct config_struct));
 		source->charset = NEWSTR(optarg);
 		break;
+	case    'u':
+		want_unix_from = 1;
+		break;
+	case    'v':
+		printf("emil version %s at your service\n", EMIL_VERSION);
+		fflush(stdout);
+		exit(EX_OK);
+		break;
 	default:
 		sprintf(ebuf,"Invalid flag: -%c",c);
 		logger(LOG_WARNING,ebuf);
 	}
    }
-  
+#ifdef SMTP  
+  if (smtp_client)
+    {
+      if (sender == NULL)
+	{
+	  fprintf(stderr, "Emil: Need sender to use SMTP.\n");
+	  exit(EX_USAGE);
+	}
+      if (recipient == NULL)
+	{
+	  fprintf(stderr, "Emil: Need recipient to use SMTP.\n");
+	  exit(EX_USAGE);
+	}
+      if (rmx == NULL)
+	{
+	  fprintf(stderr, "Emil: Need relay to use SMTP.\n");
+	  exit(EX_USAGE);
+	}
+      spawn_mailer = 0;
+      smtp_open(rmx);
+    }
+#endif
   if (sender == NULL && source == NULL)
     { 
 #ifdef DEBUG
@@ -265,7 +350,8 @@ int	main(int argc, char *argv[])
       fflush(stderr);
       exit(EX_OK);
     }
-
+  if (target == NULL)
+    target = (struct config_struct *)Yalloc(sizeof(struct config_struct));
   fix_config_struct(target);
   if (source == NULL)
     source = (struct config_struct *) getoption(sender, recipient, rmx); /* rmx?? smx */
@@ -275,7 +361,9 @@ int	main(int argc, char *argv[])
 		target ? (target->name ? target->name : "unknown") : "Unknown"
 		);
   logger(LOG_DEBUG,ebuf);
- 
+  if (source == NULL)
+    source = (struct config_struct *)Yalloc(sizeof(struct config_struct));
+
   d = (struct data *)load_data();
   logger(LOG_DEBUG,"Data loaded");
   rootmess = (struct message *)Yalloc(sizeof(struct message));
@@ -286,20 +374,30 @@ int	main(int argc, char *argv[])
   if (edebug)
     fprintf(stderr, "\n* Parsing message.\n");
 #endif
-  if (parse_message(rootmess) == NOK)
+  if (target->iformat == TRANSPARENT || parse_message(rootmess) == NOK)
     {
 #ifdef DEBUG
       if (edebug)
-	fprintf(stderr, "*** Parse message failed.\n");
+	{
+	  if (target->iformat == TRANSPARENT)
+	    fprintf(stderr, "*** TRANSPARENT Delivery ***.\n");
+	  else
+	    fprintf(stderr, "*** Parse message failed.\n");
+	}
 #endif
       det_sket_sig = TRUE;
-      logger(LOG_WARNING,"Parsing failed");
+      if (target->iformat != TRANSPARENT)
+	logger(LOG_WARNING,"Parsing failed");
       d->offset = 0;
-      if (spawn_sendmail) do_fork();
+      if (spawn_mailer) do_fork();
       	print_log();
-      	print_body(d);
-      if (spawn_sendmail) end_fork();
+      out_message(rootmess);
+      if (spawn_mailer) end_fork();
       closelog();
+#ifdef SMTP
+      if (smtp_client)
+	smtp_close(rmx);
+#endif
       exit(EX_OK);
     }
   process = 1;
@@ -316,22 +414,31 @@ int	main(int argc, char *argv[])
       det_sket_sig = TRUE;
       logger(LOG_WARNING, "Conversion failed");
       d->offset = 0;
-      if (spawn_sendmail) do_fork();
+      if (spawn_mailer) do_fork();
   	print_log();
-      	print_body(d);
-      if (spawn_sendmail) end_fork();
+      out_message(rootmess);
+      if (spawn_mailer) end_fork();
       closelog();
+#ifdef SMTP
+      if (smtp_client)
+	smtp_close(rmx);
+#endif
       exit(EX_OK);
     }
-  if (spawn_sendmail) do_fork();
+  if (spawn_mailer) do_fork();
   print_log();
 #ifdef DEBUG
   if (edebug)
     fprintf(stderr, "\n* Output message.\n");
 #endif
-
+  if (rootmess->sd->format == RFC822)
+    multipart_kludge(rootmess);
   out_message(rootmess);
-  if (spawn_sendmail) end_fork();
+#ifdef SMTP
+  if (smtp_client)
+    smtp_close(rmx);
+#endif
+  if (spawn_mailer) end_fork();
   /* end_fork returns here if not spawning: */
   closelog();
 #ifdef DEBUG
@@ -372,7 +479,16 @@ fix_config_struct(struct config_struct *c)
 #endif
 	  }
 	else
-	  {
+	  if (cmatch(c->format, "transparent") == TRUE)
+	    {
+	      c->iformat = TRANSPARENT;
+#ifdef DEBUG
+	      if (edebug)
+		fprintf(stderr, "TRANSPARENT");
+#endif
+	    }
+	  else
+	    {
 	    c->iformat = RFC822;
 #ifdef DEBUG
 	    if (edebug)
